function appendCoeffs(coeffs,filename,varargin)
% This function appends filter coefficients to the end of a file as a 
% const float array in c/c++ notation. It can handle a plain coefficient
% vecor or a struct exported from sptool containing a tf field. In that
% case, if the field tf.den is a denominator different from 1 
% (i.e. the struct is an IIR filter), it will also be written to the file.
%
% INPUT:
% coeffs ... filter coefficients to be written to a file. Either a plain
%            column vecor or a struct exported from sptool are possible.
% 
% filename ... file to append to (string, enclosed in ''). Defaults to
%              'taps.cpp' if not specified.
%
% Author: Rüdiger Fasching
% Date: October 2018

if nargin == 1
    filename = 'taps.cpp'; % set the default filename 
end

if nargin > 2
    sprintf('appendCoeffs: Too many input arguments!');
    return
end

file = fopen(filename,'a'); % append to the end of the file
if isfield(coeffs, 'tf') % check if coeffs is a struct exported from sptool
    fprintf(file, '\n\nconst float %s_num[] = {\n',inputname(1)); % create a const float array
    fprintf(file, '%2.10f,\n',coeffs.tf.num); % print the numerator
    fprintf(file, '};');
    if length(coeffs.tf.den) > 1 % check if we have a denominator other than 1
        fprintf(file, '\n\nconst float %s_den[] = {\n',inputname(1));
        fprintf(file, '%2.10f,\n',coeffs.tf.den); % print the denominator
        fprintf(file, '};');
    end
else % just print the plain coeffs variable, since its not a struct
    fprintf(file, '\n\nconst float %s[] = {\n',inputname(1));
    fprintf(file, '%2.10f,\n',coeffs);
    fprintf(file, '};');
end

fclose(file);
