/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * A BiquadChain creates a series of Biquad filter objects and forwards the
 * input through the filters
 * 
 * The Biquad filter coefficients are stored internally
 */

#include <iostream>
#include "BiquadChain.h"
#include <vector>
#include "blockprocessing.h"

BiquadChain::BiquadChain() {
    bufferin.resize(FRAMELENGTH);
    bufferout.resize(FRAMELENGTH);
};

BiquadChain::~BiquadChain() {
}

void BiquadChain::setCoefficients(const void *coeff, const int Ncoef){
    if((Ncoef-1) % NBICOEFF == 0){
        NBIQUAD = Ncoef / NBICOEFF;
        biquads.resize(NBIQUAD);

        // the pointer coeff targets the filter coefficients
        float *coeftemp = (float *) coeff;
        for(int i=0;i<NBIQUAD;i++){ // for each biquad object
            biquads[i].setCoefficients(coeftemp);
            coeftemp+=NBICOEFF; // each biquad expects NBICOEFF coefficients
        }
        gain = *(++coeftemp); // last entry is the total gain
    }else{
        NBIQUAD = 0; // no biquads
        std::cout << "There is something wrong with your Biquad coefficients" << std::endl;
    }
}

void BiquadChain::doFiltering(const void *input, void *output) {
    float *in = (float*)input;
    float *out = (float*)output;
    int i,j;
    // copy input to bufferin
    for(i=0; i<FRAMELENGTH; i++)
        bufferin[i]=gain * (*in++);
        
    // perform filtering
    for(i=0; i<NBIQUAD; i++){ // for each biquad object
        biquads[i].doFiltering(&bufferin[0],&bufferout[0]); // filter
        for (j=0; j<FRAMELENGTH; j++) // and copy the output of the current filter to the input of the next filter
            bufferin[j] = bufferout[j];
    }
    
    // finally, copy result to output
    for(i=0; i<FRAMELENGTH; i++)
        *out++ = bufferout[i];
    
    
}