/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * A FIRfilter objects performs the convolution of the provided input signal
 * using internal coefficients, based on block processing.
 */

#include <vector>
#include "blockprocessing.h"
#include "FIRfilter.h"
#include <iostream>

using std::vector;

FIRfilter::FIRfilter() { //constructor
};

FIRfilter::~FIRfilter() { //deconstructor
};

void FIRfilter::setCoefficients(const float coeff[], const int Ncoeff) {
    NFIR = Ncoeff;
    NFIRBUF = FRAMELENGTH + Ncoeff;
    buffer.resize(NFIRBUF); // reserve space for the vector objects
    coefficients.resize(NFIR);
    for(int i=0;i<NFIR;i++){ // copy coefficients to internal memory
        coefficients[i]=*coeff++;
    }
};

void FIRfilter::doFiltering(const void *in, void *out, unsigned long framelength) {
    /* Note, benchmark tests demonstrate that execution time on the RPi of 32bit and 16bit multiplications
     * is comparable. Thus, everything is done in 32bit float, beneficial to prevent overhead issues.
     */
    float *input = (float*) in;
    float *output = (float*) out;
    float acc; // accumulator
    
    for(int i=0; i<framelength; i++) // copy input to internal buffer right after the samples from the previous frame
        buffer[NFIR-1+i] = *(input+i);
    
    // the convolution sum
    for (int i = 0; i < framelength; i++) { // for each output sample y[n]
        input = &buffer[NFIR-1+i]; // set input pointer to x[n]
        acc = 0;
        for (int j = 0; j < NFIR; j++) { // and multiply a0*x[n] + a1*x[n-1] + ...
            acc += coefficients[j] * (*input); // perform multiplication in float (32bit)
            input--; // decrement input pointer
        }
        *output = acc;
        output++; // increment output pointer
    }

    // finally, copy the last NFIR elements in the current frame to the beginning of the buffer
    for(int i=0; i<NFIR-1; i++)
        buffer[i] = buffer[framelength+i];
    return;
}
