/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * A IIRfilter objects performs the convolution of the provided input signal
 * using internal coefficients, based on block processing.
 */

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include "blockprocessing.h"
#include "IIRfilter.h"

IIRfilter::IIRfilter() {
};

IIRfilter::~IIRfilter() {
};

float IIRfilter::setCoefficients(const float coeff[], const int Ncoef) {
    NIIR = Ncoef;
    NIIRBUF = FRAMELENGTH + Ncoef;
    buffer.resize(NIIRBUF);
    coefficients.resize(NIIR);
    // copy coefficients to private ones
    coefficients[0] = coeff[0]; // first entry has same sign
    for(int i=1; i<NIIR; i++)
        coefficients[i] = -coeff[i]; // all the others have negative sign
    if(coefficients[0] != 1){ // check if a0 equals one
        std::cout << "\nWarning in IIR coefficients: a0 is not 1\n";
    }
};

void IIRfilter::doFiltering(const void *in, void *out, unsigned long framelength) {
    float *input = (float*) in;
    float *output = (float*) out;
    float acc;
    int i,j;
    // fill buffer using sampling points from the previous frame buffer
    for(i=0; i<framelength; i++)
        buffer[NIIR+i] = *(input+i);
    
    // do filtering (the convolution sum)
    for (i = 0; i < framelength; i++) {
        acc = 0;
        for (j = 0; j <= NIIR; j++) {   // consider sign of coefficients
            acc += coefficients[j] * buffer[NIIR + i - j];
        }
        buffer[NIIR + i] = acc;
        *output = acc;
        output++; // write to output
    }
    // finally, save results for the upcoming frame buffer
    for(int i=0; i<NIIR; i++)
        buffer[i] = buffer[framelength+i];
    return;
}