/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * UNIT 2, part 2: FIR, IIR and Biquad Filter experiments
 * 
 * 
 */
#include <iostream>
#include <math.h>
#include "portaudio.h"
#include "blockprocessing.h"
#include "FIRfilter.h"
#include "IIRfilter.h"
#include "Biquadfilter.h"
#include "taps.cpp"
#include "BiquadChain.h"
#include <vector>

// collect objects for computation in this struct
struct UserDataExperiment{
    int checkCallback;
    FIRfilter fir; // load a FIR filter
    IIRfilter iir; // load an IIR filter
    BiquadChain bchain;  // load a chain of Biquad filters
};

// the callback function, evaluated at each interrupt
static int paCallback(const void *input, void *output,
    unsigned long framelength,
    const PaStreamCallbackTimeInfo* timeInfo,
    PaStreamCallbackFlags statusFlags,
    void *userdata)
{
    float *in = (float*)input;
    float *out = (float*)output;
    float tempstream[framelength]; // reserve space for transfering output of FIR to input of IIR
       
    UserDataExperiment *ud = (UserDataExperiment*)userdata;
    ud->checkCallback ++; // report that callback function is working,
    
    // FIR filter followed by an IIR filter
    ud->fir.doFiltering(in, tempstream, framelength);
    ud->iir.doFiltering(tempstream, out, framelength);
    
    // Biquad chain
    ud->bchain.doFiltering(in,out);

    return 0;
}

int main(int argc, char *argv[]) {   
    PaStream *stream;
    PaError err;
    int checkCallback_local = 0; // check whether callback function is still active
    int iTimecounter = 0;
    int Ncoeff; // temporary integer

    UserDataExperiment ud; // create objects
    ud.checkCallback = 0; // a watchdog if callback is still working (necessary for RPi)
    
    // load filter coefficients for the FIR filter
    Ncoeff = (int) sizeof(Exp3Num)/sizeof(*Exp3Num);
    ud.fir.setCoefficients(Exp3Num, Ncoeff);
    
    // load filter coefficients for the IIR filter
    Ncoeff = (int) sizeof(Exp3Den)/sizeof(*Exp3Den);
    ud.iir.setCoefficients(Exp3Den, Ncoeff);

    // load filter coefficients for the Biquad filter
    Ncoeff = (int) sizeof(cascaded)/sizeof(*cascaded);
    ud.bchain.setCoefficients(cascaded, Ncoeff);
    
    // the following loop stops and restarts the portaudio stream if it is broken
    do{
        err = Pa_Initialize();
        std::cout << "Initialize portaudio: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_OpenDefaultStream(&stream, 
            NINPUTCHANNELS, 
            NOUTPUTCHANNELS, 
            paFloat32, 
            FS, 
            FRAMELENGTH, 
            paCallback, 
            &ud);
        std::cout << "Open audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_StartStream(stream);
        std::cout << "Start audio stream: " << Pa_GetErrorText( err ) << std::endl;

        Pa_Sleep(TIMETOSLEEP); // wait until Stream is open
        
        // wait in this loop until DURATION is reached or callback function needs to be restarted
        while(iTimecounter < DURATION-1 && ud.checkCallback != checkCallback_local){ // there was a callback within last second
            checkCallback_local = ud.checkCallback;
            Pa_Sleep(TIMETOSLEEP); // wait a second
            iTimecounter++;
        }
        
        if(iTimecounter != DURATION-1 && ud.checkCallback != checkCallback_local){
            std::cout << "There is a problem with portaudio -> restart" << std::endl;
        }

        err = Pa_StopStream(stream);
        std::cout << "Stop audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_CloseStream(stream);
        std::cout << "Close audio stream: " << Pa_GetErrorText( err ) << std::endl;
    }while(iTimecounter < DURATION-1);

    Pa_Terminate();
    return 0;    
}
